package de.hftstuttgart.unifiedticketing.systems.github;

import de.hftstuttgart.unifiedticketing.core.Logging;
import de.hftstuttgart.unifiedticketing.core.TicketSystemBuilder;
import de.hftstuttgart.unifiedticketing.exceptions.AssertionException;

import java.util.logging.Level;
import java.util.logging.Logger;

public class GithubTicketSystemBuilder extends TicketSystemBuilder<GithubTicketSystemBuilder, GithubTicketSystem>
{
    private static final Logger logger = Logging.getLogger(GithubTicketSystemBuilder.class.getName());

    protected String acceptHeader;
    protected String apiKey;
    protected boolean https;
    protected String owner;
    protected String repo;
    protected String username;

    /**
     * starts builder process for a {@link GithubTicketSystem} instance
     */
    public GithubTicketSystemBuilder()
    {
        super();

        acceptHeader = "application/vnd.github.v3+json";
        https = true;

        logger.log(Level.FINEST, "Builder for Github System instance started");
    }

    public GithubTicketSystemBuilder withAuthentication(String username, String apiKey)
    {
        this.username = username;
        this.apiKey = apiKey;
        logger.log(Level.FINEST, "set authentication values");
        return this;
    }

    public GithubTicketSystemBuilder withHttp()
    {
        this.https = false;
        logger.log(Level.FINEST, "set http");
        return this;
    }

    public GithubTicketSystemBuilder withHttps()
    {
        this.https = true;
        logger.log(Level.FINEST, "set https");
        return this;
    }

    public GithubTicketSystemBuilder withOwner(String owner)
    {
        this.owner = owner;
        logger.log(Level.FINEST, String.format("set owner to %s", this.owner));
        return this;
    }

    public GithubTicketSystemBuilder withRepo(String repo)
    {
        this.repo = repo;
        logger.log(Level.FINEST, String.format("set repo to %s", this.repo));
        return this;
    }

    @Override
    public GithubTicketSystem build()
    {
        if (baseUrl == null
            || owner == null
            || repo == null)
        {
            String msg = "one of mandatory fields 'baseUrl', 'owner' or 'repo' missing!";
            throw new AssertionException(msg);
        }

        return new GithubTicketSystem(
            acceptHeader,
            String.format(
                "%s://%s/repos/%s/%s/issues",
                (https) ? "https" : "http",
                baseUrl,
                owner,
                repo
            ),
            username,
            apiKey
        );
    }
}
