package de.hftstuttgart.unifiedticketing.core;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.*;

public class FilterTest
{
    public Filter instance;

    @BeforeEach
    public void initBeforeEach()
    {
        instance = mock(Filter.class, withSettings().useConstructor().defaultAnswer(CALLS_REAL_METHODS));
    }

    @Test
    public void testWithSingleAssigneeId()
    {
        String value = "testId";
        Set<String> expected = new HashSet<>();
        expected.add(value);

        instance.withAssigneeId(value);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.ASSIGNEEID.name()));
    }

    @Test
    public void testWithMultipleAssigneeId()
    {
        String[] values = new String[]{"testId", "anotherone", "thirdId"};
        Set<String> expected = new HashSet<>(Arrays.asList(values));

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.ASSIGNEEID.name());
        instance.withAssigneeId(values[0]);
        Object valueAfterFirstAdd = instance.setFilters.get(Filter.FilterNames.ASSIGNEEID.name());
        Arrays.stream(values).forEach(instance::withAssigneeId);
        Arrays.stream(values).forEach(instance::withAssigneeId);
        Object valueAfterAllAdds = instance.setFilters.get(Filter.FilterNames.ASSIGNEEID.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),             // no initial instance present
            () -> assertNotNull(valueAfterFirstAdd),           // after first add there has to be one
            () -> assertSame(valueAfterFirstAdd, valueAfterAllAdds), // check the collection stayed the same
            () -> assertEquals(expected, valueAfterAllAdds)    // check the content is only in there once and complete
        );
    }

    @Test
    public void testWithSingleAssigneeName()
    {
        String value = "testName";
        Set<String> expected = new HashSet<>();
        expected.add(value);

        instance.withAssigneeName(value);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.ASSIGNEENAME.name()));
    }

    @Test
    public void testWithMultipleAssigneeName()
    {
        String[] values = new String[]{"testName", "anotherone", "thirdName"};
        Set<String> expected = new HashSet<>(Arrays.asList(values));

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.ASSIGNEENAME.name());
        instance.withAssigneeName(values[0]);
        Object valueAfterFirstAdd = instance.setFilters.get(Filter.FilterNames.ASSIGNEENAME.name());
        Arrays.stream(values).forEach(instance::withAssigneeName);
        Arrays.stream(values).forEach(instance::withAssigneeName);
        Object valueAfterAllAdds = instance.setFilters.get(Filter.FilterNames.ASSIGNEENAME.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),             // no initial instance present
            () -> assertNotNull(valueAfterFirstAdd),           // after first add there has to be one
            () -> assertSame(valueAfterFirstAdd, valueAfterAllAdds), // check the collection stayed the same
            () -> assertEquals(expected, valueAfterAllAdds)    // check the content is only in there once and complete
        );
    }

    @Test
    public void testWithDescriptionContain()
    {
        String expected = "test description";

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.DESCRIPTION_CONTAIN.name());
        instance.withDescriptionContain(expected);
        Object valueAfterAdd = instance.setFilters.get(Filter.FilterNames.DESCRIPTION_CONTAIN.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),         // no initial instance present
            () -> assertEquals(expected, valueAfterAdd)    // after add there has to be the set one
        );
    }

    @Test
    public void testWithDescriptionMatch()
    {
        String expected = "^some stupid regex number [0-9]+";

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.DESCRIPTION_MATCH.name());
        instance.withDescriptionMatch(expected);
        Object valueAfterAdd = instance.setFilters.get(Filter.FilterNames.DESCRIPTION_MATCH.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),         // no initial instance present
            () -> assertEquals(expected, valueAfterAdd)    // after add there has to be the set one
        );
    }

    @Test
    public void testWithSingleTicketId()
    {
        String value = "testId";
        Set<String> expected = new HashSet<>();
        expected.add(value);

        instance.withId(value);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.IDS.name()));
    }

    @Test
    public void testWithMultipleTicketIds()
    {
        String[] values = new String[]{"testId", "anotherone", "thirdId"};
        Set<String> expected = new HashSet<>(Arrays.asList(values));

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.IDS.name());
        instance.withId(values[0]);
        Object valueAfterFirstAdd = instance.setFilters.get(Filter.FilterNames.IDS.name());
        Arrays.stream(values).forEach(instance::withId);
        Arrays.stream(values).forEach(instance::withId);
        Object valueAfterAllAdds = instance.setFilters.get(Filter.FilterNames.IDS.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),             // no initial instance present
            () -> assertNotNull(valueAfterFirstAdd),           // after first add there has to be one
            () -> assertSame(valueAfterFirstAdd, valueAfterAllAdds), // check the collection stayed the same
            () -> assertEquals(expected, valueAfterAllAdds)    // check the content is only in there once and complete
        );
    }

    @Test
    public void testIsOpen()
    {
        instance
            .isOpen()
            .isClosed()
            .isOpen();

        assertEquals(Boolean.TRUE, instance.setFilters.get(Filter.FilterNames.OPEN.name()));
    }

    @Test
    public void testIsClosed()
    {
        instance
            .isClosed()
            .isOpen()
            .isClosed();

        assertEquals(Boolean.FALSE, instance.setFilters.get(Filter.FilterNames.OPEN.name()));
    }

    @Test
    public void testWithSingleLabel()
    {
        String value = "test";
        Set<String> expected = new HashSet<>();
        expected.add(value);

        instance.withLabel(value);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.LABELS.name()));
    }

    @Test
    public void testWithMultipleLabels()
    {
        String[] values = new String[]{"test", "To Do", "Bug", "Feature", "review"};
        Set<String> expected = new HashSet<>(Arrays.asList(values));

        Object valueBeforeFirstAdd = instance.setFilters.get(Filter.FilterNames.LABELS.name());
        instance.withLabel(values[0]);
        Object valueAfterFirstAdd = instance.setFilters.get(Filter.FilterNames.LABELS.name());
        Arrays.stream(values).forEach(instance::withLabel);
        Arrays.stream(values).forEach(instance::withLabel);
        Object valueAfterAllAdds = instance.setFilters.get(Filter.FilterNames.LABELS.name());

        assertAll(
            () -> assertNull(valueBeforeFirstAdd),             // no initial instance present
            () -> assertNotNull(valueAfterFirstAdd),           // after first add there has to be one
            () -> assertSame(valueAfterFirstAdd, valueAfterAllAdds), // check the collection stayed the same
            () -> assertEquals(expected, valueAfterAllAdds)    // check the content is only in there once and complete
        );
    }

    @Test
    public void testSetPage()
    {
        int value = 2;

        instance.setPage(value);
        assertEquals(value, instance.setFilters.get(Filter.FilterNames.PAGE.name()));
    }

    @Test
    public void testSetPageSize()
    {
        int value = 2;

        instance.setPageSize(value);
        assertEquals(value, instance.setFilters.get(Filter.FilterNames.PAGINATION.name()));
    }

    @Test
    public void testWithTitleContain()
    {
        String expected = "test title";

        instance.withTitleContain(expected);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.TITLE_CONTAINS.name()));
    }

    @Test
    public void testWithTitleMatch()
    {
        String expected = "^some stupid regex number [0-9]+";

        instance.withTitleMatch(expected);

        assertEquals(expected, instance.setFilters.get(Filter.FilterNames.TITLE_MATCH.name()));
    }

    @Test
    public void testAddToSet()
    {
        Set<String> expected = new HashSet<>(Arrays.asList("test", "another", "something"));
        Map<String, Object> store = new HashMap<>();

        expected.forEach(s -> Filter.addToSet(Filter.FilterNames.LABELS, store, s));
        assertEquals(expected, store.get(Filter.FilterNames.LABELS.name()));
    }

    @Test
    public void testInitialValues()
    {
        assertAll(
            () -> assertNotNull(instance.setFilters),
            () -> assertEquals(0, instance.setFilters.size())
        );
    }

    @Test
    public void testFluentApi()
    {
        Filter before = instance;
        Filter after = instance.setPage(1)
            .setPageSize(1)
            .withAssigneeId("1")
            .withAssigneeName("Max")
            .withDescriptionContain("Muster")
            .withDescriptionMatch("Mann")
            .withId("ID")
            .withLabel("blabber")
            .withTitleContain("hell no")
            .withTitleMatch("finally")
            .isOpen()
            .isClosed();

        assertSame(before, after);
    }
}
